import os
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
from src.forward_model import CFA
from pathlib import Path
from src.methods.baseline.reconstruct import run_reconstruction
from src.methods.brice_convers.reconstruct import run_reconstruction as run_reconstruction_brice_convers
from src.methods.brice_convers.utilities import folderExists
from src.utils import load_image, save_image, psnr, ssim

class DataLoader:
    def __init__(self, workingDirectoryPath):
        DataLoader.imagePaths =[]
        DataLoader.CFA_images = {}
        DataLoader.reconstruction_images = {}

        DataLoader.IMAGE_TYPES = (".jpg", ".jpeg", ".png", ".bmp", ".tif", ".tiff")
        DataLoader.WD_PATH = workingDirectoryPath

    def list_files(self, basePath, validExts=None, contains=None):
        # loop over the directory structure
        for (rootDir, dirNames, filenames) in os.walk(basePath):
            # loop over the filenames in the current directory
            for filename in filenames:
                # if the contains string is not none and the filename does not contain
                # the supplied string, then ignore the file
                if contains is not None and filename.find(contains) == -1:
                    continue

                # determine the file extension of the current file
                ext = filename[filename.rfind("."):].lower()

                # check to see if the file is an image and should be processed
                if validExts is None or ext.endswith(validExts):
                    # construct the path to the image and yield it
                    imagePath = os.path.join(rootDir, filename)
                    DataLoader.imagePaths.append(imagePath)

    def list_images(self, basePath, contains=None):
        # return the set of files that are valid
        DataLoader.list_files(self, basePath, validExts=DataLoader.IMAGE_TYPES, contains=contains)
        print("[INFO] There are {} images.".format(len(DataLoader.imagePaths)))

    def plot_input_images(self, rows=3, cols=3, figsize=(20, 20), max_images = 6, title="Input Images"):
        fig = plt.figure(figsize=figsize)
        fig.suptitle(title, fontsize=16)

        for i, imagePath in enumerate(DataLoader.imagePaths):
            if i >= max_images:
                break

            img = mpimg.imread(imagePath)
            fig.add_subplot(rows, cols, i+1)
            # image title
            plt.title(imagePath.split(os.path.sep)[-1])
            plt.text(0, -0.1, f"Shape of the image: {img.shape}.", fontsize=12, transform=plt.gca().transAxes)
            plt.imshow(img)

        plt.show()

    def plot_raw_transformation(self, zoom = True, specificIndex = None, rows=8, cols=3, figsize=(20, 20), max_images = 24, title="Raw Transformation"):
        fig, axs = plt.subplots(rows, cols, figsize=figsize)
        fig.suptitle(title, fontsize=16)

        for i, imagePath in enumerate(DataLoader.imagePaths):
            if i >= max_images and max_images != -1:
                break

            if specificIndex is not None:
                if i != specificIndex:
                    continue

            nameImage = Path(imagePath).stem

            if DataLoader.CFA_images.get(nameImage) is None:
                print("[ERROR] There is no CFA image for the image {}.".format(nameImage))
                continue

            img = mpimg.imread(imagePath)

            y = DataLoader.CFA_images[nameImage].direct(img)
            z = DataLoader.CFA_images[nameImage].adjoint(y)

            if specificIndex is None:
                line = 2*i
                subLine = 2*i+1
            else:
                line = 0
                subLine = 1

            axs[line, 0].imshow(img)
            axs[line, 0].set_title('Input image')
            axs[line, 1].imshow(y, cmap='gray')
            axs[line, 1].set_title('Output image')
            axs[line, 2].imshow(z)
            axs[line, 2].set_title('Adjoint image')

            if zoom:
                axs[subLine, 0].imshow(img[800:864, 450:514])
                axs[subLine, 0].set_title('Zoomed input image')
                axs[subLine, 1].imshow(y[800:864, 450:514], cmap='gray')
                axs[subLine, 1].set_title('Zoomed output image')
                axs[subLine, 2].imshow(z[800:864, 450:514])
                axs[subLine, 2].set_title('Zoomed adjoint image')

        plt.show()

    def plot_specific_raw_transformation(self, indexImage, zoom = True, rows=2, cols=3, figsize=(20, 10), title="Raw Transformation"):
        DataLoader.plot_raw_transformation(self, zoom, indexImage, rows, cols, figsize, -1, title)

    def compute_CFA_images(self, CFA_NAME):
        for i, imagePath in enumerate(DataLoader.imagePaths):
            nameImage = Path(imagePath).stem
            DataLoader.compute_CFA_image(self, CFA_NAME, nameImage, i)

        print("[INFO] There are {} CFA images.".format(len(DataLoader.CFA_images)))

    def compute_reconstruction_image(self, method, indexImage, options = None):
        if len(DataLoader.imagePaths) == 0:
            print("[ERROR] There is no image in imagePaths")
            return

        nameImage = Path(DataLoader.imagePaths[indexImage]).stem

        if DataLoader.CFA_images.get(nameImage) is None:
            print("[ERROR] There is no CFA image for the image {}.".format(nameImage))
            exit(1)

        img = DataLoader.load_image(self, indexImage)

        if method == "interpolation":
            cfa = options.get("cfa")

            if cfa is None:
                print("[ERROR] You must specify the cfa.")
                exit(1)

            DataLoader.reconstruction_images.setdefault(nameImage, run_reconstruction(DataLoader.CFA_images[nameImage].direct(img), cfa))

        if method == "menon":
            pattern = options.get("pattern")
            refining_step = options.get("refining_step")

            if (refining_step or cfa) is None:
                print("[ERROR] You must specify the cfa and the refining step.")
                exit(1)

            mask = DataLoader.CFA_images[nameImage].mask

            DataLoader.reconstruction_images.setdefault(nameImage, run_reconstruction_brice_convers(DataLoader.CFA_images[nameImage].direct(img), mask, pattern, refining_step))

    def compute_reconstruction_images(self, method, options = None):

        for i in range(len(DataLoader.imagePaths)):
            DataLoader.compute_reconstruction_image(self, method, i, options)

        print("[INFO] There are {} images which have been reconstructed.".format(len(DataLoader.reconstruction_images)))

    def plot_reconstructed_image(self, indexImage, method, rows=1, cols=2, figsize=(10, 10), title="Reconstructed Images"):
        res = DataLoader.get_reconstructed_image(self, indexImage)

        fig, axs = plt.subplots(rows, cols, figsize=figsize)
        axs[0].imshow(DataLoader.load_image(self, indexImage))
        axs[0].set_title('Original image')
        axs[1].imshow(res)
        axs[1].set_title('Reconstructed image')

        outputPath = os.path.join(DataLoader.WD_PATH, "output")
        folderExists(outputPath)

        imageName = Path(DataLoader.imagePaths[indexImage]).stem
        plotReconstructedImagePath = os.path.join(outputPath, imageName + "_" + method + ".png")
        plotCompImagePath = os.path.join(outputPath, "Compararison_" + imageName + "_" + method + ".png")

        #save_image( plotReconstructedImagePath, res)
        fig.savefig(plotCompImagePath)


    def get_reconstructed_image(self, indexImage):
        nameImage = Path(DataLoader.imagePaths[indexImage]).stem

        if DataLoader.reconstruction_images.get(nameImage) is None:
            print("[ERROR] There is no reconstruction image for the image {}.".format(nameImage))
            return

        return DataLoader.reconstruction_images[nameImage]

    def shape_of_recontructed_image(self, indexImage):
        return DataLoader.reconstruction_images[Path(DataLoader.imagePaths[indexImage]).stem].shape

    def compute_CFA_image(self, CFA_NAME, nameImage, indexImage):
        DataLoader.CFA_images.setdefault(nameImage, CFA(CFA_NAME, DataLoader.shape_of_image(self, indexImage)))

    def shape_of_image(self, indexImage):
        img = mpimg.imread(DataLoader.imagePaths[indexImage])
        return img.shape

    def load_image(self, indexImage):
        img = mpimg.imread(DataLoader.imagePaths[indexImage])
        return img
